@@ -14,14 +14,12 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.arrow.vector;
 
 import static org.apache.arrow.vector.NullCheckingForGet.NULL_CHECKING_ENABLED;
 
 import java.math.BigDecimal;
 import java.nio.ByteOrder;
-
 import org.apache.arrow.memory.ArrowBuf;
 import org.apache.arrow.memory.BufferAllocator;
 import org.apache.arrow.memory.util.MemoryUtil;
@@ -37,13 +35,13 @@ import org.apache.arrow.vector.util.DecimalUtility;
 import org.apache.arrow.vector.util.TransferPair;
 import org.apache.arrow.vector.validate.ValidateUtil;
 
-
 /**
- * Decimal256Vector implements a fixed width vector (32 bytes) of
- * decimal values which could be null. A validity buffer (bit vector) is
- * maintained to track which elements in the vector are null.
+ * Decimal256Vector implements a fixed width vector (32 bytes) of decimal values which could be
+ * null. A validity buffer (bit vector) is maintained to track which elements in the vector are
+ * null.
  */
-public final class Decimal256Vector extends BaseFixedWidthVector {
+public final class Decimal256Vector extends BaseFixedWidthVector
+    implements ValueIterableVector<BigDecimal> {
   public static final int MAX_PRECISION = 76;
   public static final byte TYPE_WIDTH = 32;
   private static final boolean LITTLE_ENDIAN = ByteOrder.nativeOrder() == ByteOrder.LITTLE_ENDIAN;
@@ -52,20 +50,20 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   private final int scale;
 
   /**
-   * Instantiate a Decimal256Vector. This doesn't allocate any memory for
-   * the data in vector.
+   * Instantiate a Decimal256Vector. This doesn't allocate any memory for the data in vector.
    *
    * @param name name of the vector
    * @param allocator allocator for memory management.
    */
-  public Decimal256Vector(String name, BufferAllocator allocator,
-                               int precision, int scale) {
-    this(name, FieldType.nullable(new ArrowType.Decimal(precision, scale, /*bitWidth=*/TYPE_WIDTH * 8)), allocator);
+  public Decimal256Vector(String name, BufferAllocator allocator, int precision, int scale) {
+    this(
+        name,
+        FieldType.nullable(new ArrowType.Decimal(precision, scale, /*bitWidth=*/ TYPE_WIDTH * 8)),
+        allocator);
   }
 
   /**
-   * Instantiate a Decimal256Vector. This doesn't allocate any memory for
-   * the data in vector.
+   * Instantiate a Decimal256Vector. This doesn't allocate any memory for the data in vector.
    *
    * @param name name of the vector
    * @param fieldType type of Field materialized by this vector
@@ -76,8 +74,7 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Instantiate a Decimal256Vector. This doesn't allocate any memory for
-   * the data in vector.
+   * Instantiate a Decimal256Vector. This doesn't allocate any memory for the data in vector.
    *
    * @param field field materialized by this vector
    * @param allocator allocator for memory management.
@@ -95,8 +92,7 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Get minor type for this vector. The vector holds values belonging
-   * to a particular type.
+   * Get minor type for this vector. The vector holds values belonging to a particular type.
    *
    * @return {@link org.apache.arrow.vector.types.Types.MinorType}
    */
@@ -105,18 +101,16 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
     return MinorType.DECIMAL256;
   }
 
-
   /*----------------------------------------------------------------*
-   |                                                                |
-   |          vector value retrieval methods                        |
-   |                                                                |
-   *----------------------------------------------------------------*/
-
+  |                                                                |
+  |          vector value retrieval methods                        |
+  |                                                                |
+  *----------------------------------------------------------------*/
 
   /**
    * Get the element at the given index from the vector.
    *
-   * @param index   position of element
+   * @param index position of element
    * @return element at given index
    */
   public ArrowBuf get(int index) throws IllegalStateException {
@@ -127,11 +121,10 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Get the element at the given index from the vector and
-   * sets the state in holder. If element at given index
-   * is null, holder.isSet will be zero.
+   * Get the element at the given index from the vector and sets the state in holder. If element at
+   * given index is null, holder.isSet will be zero.
    *
-   * @param index   position of element
+   * @param index position of element
    */
   public void get(int index, NullableDecimal256Holder holder) {
     if (isSet(index) == 0) {
@@ -148,7 +141,7 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   /**
    * Same as {@link #get(int)}.
    *
-   * @param index   position of element
+   * @param index position of element
    * @return element at given index
    */
   @Override
@@ -163,40 +156,34 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   /**
    * Same as {@link #getObject(int)}, but does not check for null.
    *
-   * @param index   position of element
+   * @param index position of element
    * @return element at given index
    */
   public BigDecimal getObjectNotNull(int index) {
     return DecimalUtility.getBigDecimalFromArrowBuf(valueBuffer, index, scale, TYPE_WIDTH);
   }
 
-  /**
-   * Return precision for the decimal value.
-   */
+  /** Return precision for the decimal value. */
   public int getPrecision() {
     return precision;
   }
 
-  /**
-   * Return scale for the decimal value.
-   */
+  /** Return scale for the decimal value. */
   public int getScale() {
     return scale;
   }
 
-
   /*----------------------------------------------------------------*
-   |                                                                |
-   |          vector value setter methods                           |
-   |                                                                |
-   *----------------------------------------------------------------*/
-
+  |                                                                |
+  |          vector value setter methods                           |
+  |                                                                |
+  *----------------------------------------------------------------*/
 
   /**
    * Set the element at the given index to the given value.
    *
-   * @param index    position of element
-   * @param buffer   ArrowBuf containing decimal value.
+   * @param index position of element
+   * @param buffer ArrowBuf containing decimal value.
    */
   public void set(int index, ArrowBuf buffer) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -204,16 +191,15 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Set the decimal element at given index to the provided array of bytes.
-   * Decimal256 is now implemented as Native Endian. This API allows the user
-   * to pass a decimal value in the form of byte array in BE byte order.
+   * Set the decimal element at given index to the provided array of bytes. Decimal256 is now
+   * implemented as Native Endian. This API allows the user to pass a decimal value in the form of
+   * byte array in BE byte order.
    *
-   * <p>Consumers of Arrow code can use this API instead of first swapping
-   * the source bytes (doing a write and read) and then finally writing to
-   * ArrowBuf of decimal vector.
+   * <p>Consumers of Arrow code can use this API instead of first swapping the source bytes (doing a
+   * write and read) and then finally writing to ArrowBuf of decimal vector.
    *
-   * <p>This method takes care of adding the necessary padding if the length
-   * of byte array is less than 32 (length of decimal type).
+   * <p>This method takes care of adding the necessary padding if the length of byte array is less
+   * than 32 (length of decimal type).
    *
    * @param index position of element
    * @param value array of bytes containing decimal in big endian byte order.
@@ -227,13 +213,13 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
 
     long outAddress = valueBuffer.memoryAddress() + (long) index * TYPE_WIDTH;
     if (length == 0) {
-      MemoryUtil.UNSAFE.setMemory(outAddress, Decimal256Vector.TYPE_WIDTH, (byte) 0);
+      MemoryUtil.setMemory(outAddress, Decimal256Vector.TYPE_WIDTH, (byte) 0);
       return;
     }
     if (LITTLE_ENDIAN) {
       // swap bytes to convert BE to LE
       for (int byteIdx = 0; byteIdx < length; ++byteIdx) {
-        MemoryUtil.UNSAFE.putByte(outAddress + byteIdx, value[length - 1 - byteIdx]);
+        MemoryUtil.putByte(outAddress + byteIdx, value[length - 1 - byteIdx]);
       }
 
       if (length == TYPE_WIDTH) {
@@ -243,21 +229,17 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
       if (length < TYPE_WIDTH) {
         // sign extend
         final byte pad = (byte) (value[0] < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress + length, Decimal256Vector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress + length, Decimal256Vector.TYPE_WIDTH - length, pad);
         return;
       }
     } else {
       if (length <= TYPE_WIDTH) {
         // copy data from value to outAddress
-        MemoryUtil.UNSAFE.copyMemory(
-                value,
-                MemoryUtil.BYTE_ARRAY_BASE_OFFSET,
-                null,
-                outAddress + Decimal256Vector.TYPE_WIDTH - length,
-                length);
+        MemoryUtil.copyToMemory(
+            value, 0, outAddress + Decimal256Vector.TYPE_WIDTH - length, length);
         // sign extend
         final byte pad = (byte) (value[0] < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress, Decimal256Vector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress, Decimal256Vector.TYPE_WIDTH - length, pad);
         return;
       }
     }
@@ -268,9 +250,9 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   /**
    * Set the element at the given index to the given value.
    *
-   * @param index    position of element
-   * @param start    start index of data in the buffer
-   * @param buffer   ArrowBuf containing decimal value.
+   * @param index position of element
+   * @param start start index of data in the buffer
+   * @param buffer ArrowBuf containing decimal value.
    */
   public void set(int index, long start, ArrowBuf buffer) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -279,6 +261,7 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
 
   /**
    * Sets the element at given index using the buffer whose size maybe <= 32 bytes.
+   *
    * @param index index to write the decimal to
    * @param start start of value in the buffer
    * @param buffer contains the decimal in native endian bytes
@@ -295,27 +278,27 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
     long inAddress = buffer.memoryAddress() + start;
     long outAddress = valueBuffer.memoryAddress() + (long) index * TYPE_WIDTH;
     if (LITTLE_ENDIAN) {
-      MemoryUtil.UNSAFE.copyMemory(inAddress, outAddress, length);
+      MemoryUtil.copyMemory(inAddress, outAddress, length);
       // sign extend
       if (length < TYPE_WIDTH) {
-        byte msb = MemoryUtil.UNSAFE.getByte(inAddress + length - 1);
+        byte msb = MemoryUtil.getByte(inAddress + length - 1);
         final byte pad = (byte) (msb < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress + length, Decimal256Vector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress + length, Decimal256Vector.TYPE_WIDTH - length, pad);
       }
     } else {
-      MemoryUtil.UNSAFE.copyMemory(inAddress, outAddress + Decimal256Vector.TYPE_WIDTH - length, length);
+      MemoryUtil.copyMemory(inAddress, outAddress + Decimal256Vector.TYPE_WIDTH - length, length);
       // sign extend
       if (length < TYPE_WIDTH) {
-        byte msb = MemoryUtil.UNSAFE.getByte(inAddress);
+        byte msb = MemoryUtil.getByte(inAddress);
         final byte pad = (byte) (msb < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress, Decimal256Vector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress, Decimal256Vector.TYPE_WIDTH - length, pad);
       }
     }
   }
 
-
   /**
    * Sets the element at given index using the buffer whose size maybe <= 32 bytes.
+   *
    * @param index index to write the decimal to
    * @param start start of value in the buffer
    * @param buffer contains the decimal in big endian bytes
@@ -335,22 +318,22 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
     if (LITTLE_ENDIAN) {
       // swap bytes to convert BE to LE
       for (int byteIdx = 0; byteIdx < length; ++byteIdx) {
-        byte val = MemoryUtil.UNSAFE.getByte((inAddress + length - 1) - byteIdx);
-        MemoryUtil.UNSAFE.putByte(outAddress + byteIdx, val);
+        byte val = MemoryUtil.getByte((inAddress + length - 1) - byteIdx);
+        MemoryUtil.putByte(outAddress + byteIdx, val);
       }
       // sign extend
       if (length < 32) {
-        byte msb = MemoryUtil.UNSAFE.getByte(inAddress);
+        byte msb = MemoryUtil.getByte(inAddress);
         final byte pad = (byte) (msb < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress + length, Decimal256Vector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress + length, Decimal256Vector.TYPE_WIDTH - length, pad);
       }
     } else {
-      MemoryUtil.UNSAFE.copyMemory(inAddress, outAddress + Decimal256Vector.TYPE_WIDTH - length, length);
+      MemoryUtil.copyMemory(inAddress, outAddress + Decimal256Vector.TYPE_WIDTH - length, length);
       // sign extend
       if (length < TYPE_WIDTH) {
-        byte msb = MemoryUtil.UNSAFE.getByte(inAddress);
+        byte msb = MemoryUtil.getByte(inAddress);
         final byte pad = (byte) (msb < 0 ? 0xFF : 0x00);
-        MemoryUtil.UNSAFE.setMemory(outAddress, Decimal256Vector.TYPE_WIDTH - length, pad);
+        MemoryUtil.setMemory(outAddress, Decimal256Vector.TYPE_WIDTH - length, pad);
       }
     }
   }
@@ -358,8 +341,8 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   /**
    * Set the element at the given index to the given value.
    *
-   * @param index   position of element
-   * @param value   BigDecimal containing decimal value.
+   * @param index position of element
+   * @param value BigDecimal containing decimal value.
    */
   public void set(int index, BigDecimal value) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -370,8 +353,8 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   /**
    * Set the element at the given index to the given value.
    *
-   * @param index   position of element
-   * @param value   long value.
+   * @param index position of element
+   * @param value long value.
    */
   public void set(int index, long value) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -379,12 +362,11 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Set the element at the given index to the value set in data holder.
-   * If the value in holder is not indicated as set, element in the
-   * at the given index will be null.
+   * Set the element at the given index to the value set in data holder. If the value in holder is
+   * not indicated as set, element in the at the given index will be null.
    *
-   * @param index   position of element
-   * @param holder  nullable data holder for value of element
+   * @param index position of element
+   * @param holder nullable data holder for value of element
    */
   public void set(int index, NullableDecimal256Holder holder) throws IllegalArgumentException {
     if (holder.isSet < 0) {
@@ -400,8 +382,8 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   /**
    * Set the element at the given index to the value set in data holder.
    *
-   * @param index   position of element
-   * @param holder  data holder for value of element
+   * @param index position of element
+   * @param holder data holder for value of element
    */
   public void set(int index, Decimal256Holder holder) {
     BitVectorHelper.setBit(validityBuffer, index);
@@ -409,12 +391,11 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, ArrowBuf)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, ArrowBuf)} except that it handles the case when index is greater than
+   * or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param buffer  ArrowBuf containing decimal value.
+   * @param index position of element
+   * @param buffer ArrowBuf containing decimal value.
    */
   public void setSafe(int index, ArrowBuf buffer) {
     handleSafe(index);
@@ -422,9 +403,8 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #setBigEndian(int, byte[])} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #setBigEndian(int, byte[])} except that it handles the case when index is
+   * greater than or equal to existing value capacity {@link #getValueCapacity()}.
    */
   public void setBigEndianSafe(int index, byte[] value) {
     handleSafe(index);
@@ -432,13 +412,12 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, int, ArrowBuf)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, int, ArrowBuf)} except that it handles the case when index is greater
+   * than or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index    position of element
-   * @param start    start index of data in the buffer
-   * @param buffer   ArrowBuf containing decimal value.
+   * @param index position of element
+   * @param start start index of data in the buffer
+   * @param buffer ArrowBuf containing decimal value.
    */
   public void setSafe(int index, long start, ArrowBuf buffer) {
     handleSafe(index);
@@ -446,12 +425,11 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, BigDecimal)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, BigDecimal)} except that it handles the case when index is greater
+   * than or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param value   BigDecimal containing decimal value.
+   * @param index position of element
+   * @param value BigDecimal containing decimal value.
    */
   public void setSafe(int index, BigDecimal value) {
     handleSafe(index);
@@ -459,12 +437,11 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, long)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, long)} except that it handles the case when index is greater than or
+   * equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param value   long value.
+   * @param index position of element
+   * @param value long value.
    */
   public void setSafe(int index, long value) {
     handleSafe(index);
@@ -472,12 +449,11 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, NullableDecimalHolder)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, NullableDecimalHolder)} except that it handles the case when index is
+   * greater than or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param holder  nullable data holder for value of element
+   * @param index position of element
+   * @param holder nullable data holder for value of element
    */
   public void setSafe(int index, NullableDecimal256Holder holder) throws IllegalArgumentException {
     handleSafe(index);
@@ -485,12 +461,11 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #set(int, Decimal256Holder)} except that it handles the
-   * case when index is greater than or equal to existing
-   * value capacity {@link #getValueCapacity()}.
+   * Same as {@link #set(int, Decimal256Holder)} except that it handles the case when index is
+   * greater than or equal to existing value capacity {@link #getValueCapacity()}.
    *
-   * @param index   position of element
-   * @param holder  data holder for value of element
+   * @param index position of element
+   * @param holder data holder for value of element
    */
   public void setSafe(int index, Decimal256Holder holder) {
     handleSafe(index);
@@ -498,8 +473,8 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Store the given value at a particular position in the vector. isSet indicates
-   * whether the value is NULL or not.
+   * Store the given value at a particular position in the vector. isSet indicates whether the value
+   * is NULL or not.
    *
    * @param index position of the new value
    * @param isSet 0 for NULL value, 1 otherwise
@@ -515,9 +490,8 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Same as {@link #setSafe(int, int, int, ArrowBuf)} except that it handles
-   * the case when the position of new value is beyond the current value
-   * capacity of the vector.
+   * Same as {@link #setSafe(int, int, int, ArrowBuf)} except that it handles the case when the
+   * position of new value is beyond the current value capacity of the vector.
    *
    * @param index position of the new value
    * @param isSet 0 for NULL value, 1 otherwise
@@ -534,23 +508,27 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
     for (int i = 0; i < getValueCount(); ++i) {
       BigDecimal value = getObject(i);
       if (value != null) {
-        ValidateUtil.validateOrThrow(DecimalUtility.checkPrecisionAndScaleNoThrow(value, getPrecision(), getScale()),
-            "Invalid value for Decimal256Vector at position " + i + ". Value does not fit in precision " +
-                getPrecision() + " and scale " + getScale() + ".");
+        ValidateUtil.validateOrThrow(
+            DecimalUtility.checkPrecisionAndScaleNoThrow(value, getPrecision(), getScale()),
+            "Invalid value for Decimal256Vector at position "
+                + i
+                + ". Value does not fit in precision "
+                + getPrecision()
+                + " and scale "
+                + getScale()
+                + ".");
       }
     }
   }
 
   /*----------------------------------------------------------------*
-   |                                                                |
-   |                      vector transfer                           |
-   |                                                                |
-   *----------------------------------------------------------------*/
-
+  |                                                                |
+  |                      vector transfer                           |
+  |                                                                |
+  *----------------------------------------------------------------*/
 
   /**
-   * Construct a TransferPair comprising this and a target vector of
-   * the same type.
+   * Construct a TransferPair comprising this and a target vector of the same type.
    *
    * @param ref name of the target vector
    * @param allocator allocator for the target vector
@@ -562,8 +540,7 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
   }
 
   /**
-   * Construct a TransferPair comprising this and a target vector of
-   * the same type.
+   * Construct a TransferPair comprising this and a target vector of the same type.
    *
    * @param field Field object used by the target vector
    * @param allocator allocator for the target vector
@@ -589,8 +566,9 @@ public final class Decimal256Vector extends BaseFixedWidthVector {
     Decimal256Vector to;
 
     public TransferImpl(String ref, BufferAllocator allocator) {
-      to = new Decimal256Vector(ref, allocator, Decimal256Vector.this.precision,
-              Decimal256Vector.this.scale);
+      to =
+          new Decimal256Vector(
+              ref, allocator, Decimal256Vector.this.precision, Decimal256Vector.this.scale);
     }
 
     public TransferImpl(Field field, BufferAllocator allocator) {
