@@ -14,18 +14,18 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.arrow.vector;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertNull;
-import static org.junit.Assert.assertSame;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertSame;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 import java.util.ArrayList;
+import java.util.Collections;
+import java.util.List;
 import java.util.Map;
-
 import org.apache.arrow.memory.ArrowBuf;
 import org.apache.arrow.memory.BufferAllocator;
 import org.apache.arrow.vector.complex.MapVector;
@@ -37,23 +37,24 @@ import org.apache.arrow.vector.complex.writer.BaseWriter.ListWriter;
 import org.apache.arrow.vector.complex.writer.BaseWriter.MapWriter;
 import org.apache.arrow.vector.types.Types.MinorType;
 import org.apache.arrow.vector.types.pojo.ArrowType;
+import org.apache.arrow.vector.types.pojo.Field;
 import org.apache.arrow.vector.types.pojo.FieldType;
 import org.apache.arrow.vector.util.JsonStringArrayList;
 import org.apache.arrow.vector.util.TransferPair;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 public class TestMapVector {
 
   private BufferAllocator allocator;
 
-  @Before
+  @BeforeEach
   public void init() {
     allocator = new DirtyRootAllocator(Long.MAX_VALUE, (byte) 100);
   }
 
-  @After
+  @AfterEach
   public void terminate() throws Exception {
     allocator.close();
   }
@@ -90,7 +91,7 @@ public class TestMapVector {
         mapReader.setPosition(i);
         for (int j = 0; j < i + 1; j++) {
           mapReader.next();
-          assertEquals("record: " + i, j, mapReader.key().readLong().longValue());
+          assertEquals(j, mapReader.key().readLong().longValue(), "record: " + i);
           assertEquals(j, mapReader.value().readInteger().intValue());
         }
       }
@@ -136,7 +137,7 @@ public class TestMapVector {
           } else {
             for (int j = 0; j < i + 1; j++) {
               mapReader.next();
-              assertEquals("record: " + i, j, mapReader.key().readLong().longValue());
+              assertEquals(j, mapReader.key().readLong().longValue(), "record: " + i);
               if (i == 5) {
                 assertFalse(mapReader.value().isSet());
               } else {
@@ -152,7 +153,7 @@ public class TestMapVector {
   @Test
   public void testCopyFrom() throws Exception {
     try (MapVector inVector = MapVector.empty("input", allocator, false);
-         MapVector outVector = MapVector.empty("output", allocator, false)) {
+        MapVector outVector = MapVector.empty("output", allocator, false)) {
       UnionMapWriter writer = inVector.getWriter();
       writer.allocate();
 
@@ -194,12 +195,11 @@ public class TestMapVector {
 
       // assert the output vector is correct
       FieldReader reader = outVector.getReader();
-      assertTrue("shouldn't be null", reader.isSet());
+      assertTrue(reader.isSet(), "shouldn't be null");
       reader.setPosition(1);
-      assertFalse("should be null", reader.isSet());
+      assertFalse(reader.isSet(), "should be null");
       reader.setPosition(2);
-      assertTrue("shouldn't be null", reader.isSet());
-
+      assertTrue(reader.isSet(), "shouldn't be null");
 
       /* index 0 */
       Object result = outVector.getObject(0);
@@ -455,20 +455,26 @@ public class TestMapVector {
           StructVector dataVector1 = (StructVector) toVector.getDataVector();
 
           for (int i = 0; i < splitLength; i++) {
-            dataLength1 = offsetBuffer.getInt((start + i + 1) * MapVector.OFFSET_WIDTH) -
-                    offsetBuffer.getInt((start + i) * MapVector.OFFSET_WIDTH);
-            dataLength2 = toOffsetBuffer.getInt((i + 1) * MapVector.OFFSET_WIDTH) -
-                    toOffsetBuffer.getInt(i * MapVector.OFFSET_WIDTH);
+            dataLength1 =
+                offsetBuffer.getInt((start + i + 1) * MapVector.OFFSET_WIDTH)
+                    - offsetBuffer.getInt((start + i) * MapVector.OFFSET_WIDTH);
+            dataLength2 =
+                toOffsetBuffer.getInt((i + 1) * MapVector.OFFSET_WIDTH)
+                    - toOffsetBuffer.getInt(i * MapVector.OFFSET_WIDTH);
 
-            assertEquals("Different data lengths at index: " + i + " and start: " + start,
-                    dataLength1, dataLength2);
+            assertEquals(
+                dataLength1,
+                dataLength2,
+                "Different data lengths at index: " + i + " and start: " + start);
 
             offset1 = offsetBuffer.getInt((start + i) * MapVector.OFFSET_WIDTH);
             offset2 = toOffsetBuffer.getInt(i * MapVector.OFFSET_WIDTH);
 
             for (int j = 0; j < dataLength1; j++) {
-              assertEquals("Different data at indexes: " + offset1 + " and " + offset2,
-                      dataVector.getObject(offset1), dataVector1.getObject(offset2));
+              assertEquals(
+                  dataVector.getObject(offset1),
+                  dataVector1.getObject(offset2),
+                  "Different data at indexes: " + offset1 + " and " + offset2);
 
               offset1++;
               offset2++;
@@ -634,11 +640,12 @@ public class TestMapVector {
       MapWriter valueWriter;
 
       // we are essentially writing Map<Long, Map<Long, Long>>
-      // populate map vector with the following three records
+      // populate map vector with the following four records
       // [
       //    null,
       //    [1:[50: 100, 200:400], 2:[75: 175, 150: 250]],
-      //    [3:[10: 20], 4:[15: 20], 5:[25: 30, 35: null]]
+      //    [3:[10: 20], 4:[15: 20], 5:[25: 30, 35: null]],
+      //    [8:[15: 30, 10: 20]]
       // ]
 
       /* write null at index 0 */
@@ -700,11 +707,26 @@ public class TestMapVector {
 
       mapWriter.endMap();
 
-      assertEquals(2, mapVector.getLastSet());
+      /* write one or more maps at index 3 */
+      mapWriter.setPosition(3);
+      mapWriter.startMap();
 
-      mapWriter.setValueCount(3);
+      mapWriter.startEntry();
+      mapWriter.key().bigInt().writeBigInt(8);
+      valueWriter = mapWriter.value().map();
+      valueWriter.startMap();
+      writeEntry(valueWriter, 15, 30L);
+      writeEntry(valueWriter, 10, 20L);
+      valueWriter.endMap();
+      mapWriter.endEntry();
 
-      assertEquals(3, mapVector.getValueCount());
+      mapWriter.endMap();
+
+      assertEquals(3, mapVector.getLastSet());
+
+      mapWriter.setValueCount(4);
+
+      assertEquals(4, mapVector.getValueCount());
 
       // Get mapVector element at index 0
       Object result = mapVector.getObject(0);
@@ -778,19 +800,40 @@ public class TestMapVector {
       assertEquals(35L, getResultKey(innerMap));
       assertNull(innerMap.get(MapVector.VALUE_NAME));
 
+      // Get mapVector element at index 3
+      result = mapVector.getObject(3);
+      resultSet = (ArrayList<?>) result;
+
+      // only 1 map entry at index 3
+      assertEquals(1, resultSet.size());
+
+      resultStruct = (Map<?, ?>) resultSet.get(0);
+      assertEquals(8L, getResultKey(resultStruct));
+      list = (ArrayList<Map<?, ?>>) getResultValue(resultStruct);
+      assertEquals(2, list.size()); // value is a list of 2 maps
+      innerMap = list.get(0);
+      assertEquals(15L, getResultKey(innerMap));
+      assertEquals(30L, getResultValue(innerMap));
+      innerMap = list.get(1);
+      assertEquals(10L, getResultKey(innerMap));
+      assertEquals(20L, getResultValue(innerMap));
+
       /* check underlying bitVector */
       assertTrue(mapVector.isNull(0));
       assertFalse(mapVector.isNull(1));
       assertFalse(mapVector.isNull(2));
+      assertFalse(mapVector.isNull(3));
 
       /* check underlying offsets */
       final ArrowBuf offsetBuffer = mapVector.getOffsetBuffer();
 
-      /* mapVector has 0 entries at index 0, 2 entries at index 1, and 3 entries at index 2 */
+      // mapVector has 0 entries at index 0, 2 entries at index 1, 3 entries at index 2,
+      // and 1 entry at index 3
       assertEquals(0, offsetBuffer.getInt(0 * MapVector.OFFSET_WIDTH));
       assertEquals(0, offsetBuffer.getInt(1 * MapVector.OFFSET_WIDTH));
       assertEquals(2, offsetBuffer.getInt(2 * MapVector.OFFSET_WIDTH));
       assertEquals(5, offsetBuffer.getInt(3 * MapVector.OFFSET_WIDTH));
+      assertEquals(6, offsetBuffer.getInt(4 * MapVector.OFFSET_WIDTH));
     }
   }
 
@@ -806,7 +849,8 @@ public class TestMapVector {
       // populate map vector with the following two records
       // [
       //    [[5: 10, 20: 40]:[50: 100, 200: 400], [50: 100]:[75: 175, 150: 250]],
-      //    [[1: 2]:[10: 20], [30: 40]:[15: 20], [50: 60, 70: null]:[25: 30, 35: null], [5: null]: null]
+      //    [[1: 2]:[10: 20], [30: 40]:[15: 20], [50: 60, 70: null]:[25: 30, 35: null], [5: null]:
+      // null]
       // ]
 
       mapWriter.setPosition(0);
@@ -1178,4 +1222,23 @@ public class TestMapVector {
       toVector.clear();
     }
   }
+
+  @Test
+  public void testMakeTransferPairPreserveNullability() {
+    Field intField = new Field("int", FieldType.notNullable(MinorType.INT.getType()), null);
+    List<Field> fields = Collections.singletonList(intField);
+    Field structField =
+        new Field("struct", FieldType.notNullable(ArrowType.Struct.INSTANCE), fields);
+    Field structField2 =
+        new Field("struct", FieldType.notNullable(ArrowType.Struct.INSTANCE), fields);
+    FieldVector vec = structField.createVector(allocator);
+
+    TransferPair tp = vec.getTransferPair(structField2, allocator);
+    tp.transfer();
+
+    FieldVector res = (FieldVector) tp.getTo();
+
+    assertEquals(intField, vec.getField().getChildren().get(0));
+    assertEquals(intField, res.getField().getChildren().get(0));
+  }
 }
